/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * VarEval application
 * 
 * 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>

/* Maximum length of any variable */
#define MAXRESULT	1024

static char *tdesc = "Return the content of  dynamic variable";

static char *app_vareval = "VarEval";

static char *vareval_synopsis = "VarEval(newvar=somevarname)";

static char *vareval_descrip =
"VarEval(newvar=somevarstring)\n"
"Returns the content of variable somevarstring in newvar\n"
"\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int vareval_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char *s, *newvar=NULL, tmp[MAXRESULT];
	char *varn, *varn2;

	LOCAL_USER_ADD(u);

	/* Check and parse arguments */
	if (data) {
		s = ast_strdupa((char *)data);
		if (s) {
			newvar = strsep(&s, "=");
			if (newvar && (newvar[0] != '\0')) {
				memset(tmp, 0, MAXRESULT);
	
				varn = pbx_builtin_getvar_helper(chan, s);
				if (varn) {
					pbx_substitute_variables_helper(chan, varn, tmp, MAXRESULT - 1);
					pbx_builtin_setvar_helper(chan, newvar, tmp);
				}
			}
		} else {
			ast_log(LOG_ERROR, "Out of memory\n");
			res = -1;
		}
	}

	LOCAL_USER_REMOVE(u);
	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app_vareval);
}

int load_module(void)
{
	return ast_register_application(app_vareval, vareval_exec, vareval_synopsis, vareval_descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
